"""
these special ingredients are good for applying their cook method
to create a B/W mask for other ingredients to use internally
"""

from .seasoning import Seasoning
from .threshold import Threshold
from .rectangle import Rectangle
