"""
image and animation processing framework

pyrogis.ingredients -> Ingredient types

pyrogis.kitchen -> cooking "runtime"

pyrogis.restaurant -> feedback/terminal interface handler

pyrogis.course -> animation wrapper
"""
